#!/usr/bin/env python
# -*- coding:utf-8 -*-


"""
Simple ILP with predicate constraints to avoid collision in solution

Copyright 2021 aferro (ORCID: 0000-0003-3830-3595)
"""


from time import time
#
import numpy as np
import cvxpy as cp
from cvxpy.settings import CPLEX


FLOAT_DTYPE = np.float64


# objective
x = cp.Variable(7, boolean=True)
c = FLOAT_DTYPE([-3, -3.5, -3.1, -3.3, -2.5, -3, 0])
# inequality constraints
G = FLOAT_DTYPE([[1000, 1500, 1200, 1350, 900, 1300, 0],
                 [1, 1, 0, 0, 0, 0, -1]])
h = FLOAT_DTYPE([5000, 1])
# equality constraints
A = FLOAT_DTYPE([[0, 0, 0, 0, 0, 0, 1]])
b = FLOAT_DTYPE([0])

# solver
prob = cp.Problem(cp.Minimize(c.T@x),
                  [G@x <= h, A@x == b])
t = time()
score = prob.solve()
elapsed_s = time() - t
# solution
print("status:", prob.status)  # optimal means solution found
print("story points:", -prob.value)  # optimal value for -<c, x>
print("money:", G[0]@x.value)
print("optimal x:", x.value)
print("opt duration (seconds):", elapsed_s)
